@@ -1,58 +1,46 @@
-#=Copyright Infomation
-#==========================================================
-#Module Name       : Religion::Islam::Qibla
-#Program Author   : Dr. Ahmed Amin Elsheshtawy, Ph.D. Physics, E.E.
-#Home Page           : http://www.islamware.com, http://www.mewsoft.com
-#Contact Email      : support@islamware.com, support@mewsoft.com
-#Copyrights  2006-2013 IslamWare. All rights reserved.
-#==========================================================
+#   Copyright Infomation
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+# Author : Dr. Ahmed Amin Elsheshtawy, Ph.D.
+# Website: http://www.mewsoft.com
+# Email  : support@mewsoft.com
+# Copyrights (c) 2000-2015 Mewsoft Corporation. All rights reserved.
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 package Religion::Islam::Qibla;
 
 use Carp;
 use strict;
 use warnings;
+use Math::Trig;
 
-our $VERSION = '2.0';
-
-use constant PI => 4 * atan2(1, 1);	#3.1415926535897932 PI=22/7, Pi = Atn(1) * 4
-use constant DtoR => PI / 180;	# Degree to Radians
-use constant RtoD => 180 / PI;	# Radians to Degrees
-#==========================================================
-#==========================================================
+our $VERSION = '3.0';
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 sub new {
-my ($class, %args) = @_;
-    
+    my ($class, %args) = @_;
 	my $self = bless {}, $class;
 	# Default destination point is the  Kabah Lat=21 Deg N, Long 40 Deg E
-	$self->{DestLat} = $args{DestLat}? $args{DestLat}: 21;
-	$self->{DestLong} = $args{DestLong}? $args{DestLong}: 40;
+	$self->{DestLat} = $args{DestLat}? $args{DestLat}: 21.423333; # 21.423333;
+	$self->{DestLong} = $args{DestLong}? $args{DestLong}: 39.823333; # 39.823333;
     return $self;
 }
-#==========================================================
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 sub DestLat {
 my ($self) = shift; 
 	$self->{DestLat} = shift if @_;
 	return $self->{DestLat};
 }
-#==========================================================
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 sub DestLong {
 my ($self) = shift; 
 	$self->{DestLong} = shift if @_;
 	return $self->{DestLong};
 }
-#==========================================================
-#Inverse Cosine, ArcCos
-sub acos {
-my ($self, $x) = @_; 
-	return ($x<-1 or $x>1) ? undef : (atan2(sqrt(1-$x*$x),$x) ); 
-}
-#==========================================================
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #Converting from Degrees, Minutes and Seconds to Decimal Degrees
 sub DegreeToDecimal {
 my ($self, $degrees, $minutes, $seconds) = @_;
 	return $degrees + ($minutes / 60) + ($seconds / 3600);
 }
-#==========================================================
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #Converting from Decimal Degrees to Degrees, Minutes and Seconds
 sub DecimalToDegree {
 my ($self, $decimal_degree) = @_;
@@ -64,7 +52,7 @@ my ($degrees, $minutes, $seconds, $ff);
     $seconds = 60 * ((60 * $ff) - $minutes);
 	return ($degrees, $minutes, $seconds);
 }
-#==========================================================
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 # The shortest distance between points 1 and 2 on the earth's surface is
 # d = arccos{cos(Dlat) - [1 - cos(Dlong)]cos(lat1)cos(lat2)}
 # Dlat = lab - lat2
@@ -90,16 +78,16 @@ sub  GreatCircleDistance {
 my ($self, $orig_lat , $dest_lat, $orig_long, $dest_long) = @_;
 my ($d, $l1, $l2, $i1, $i2);
     
-    $l1 = $orig_lat * DtoR;
-    $l2 = $dest_lat * DtoR;
-    $i1 = $orig_long * DtoR;
-    $i2 = $dest_long * DtoR;
+    $l1 = deg2rad($orig_lat);
+    $l2 = deg2rad($dest_lat);
+    $i1 = deg2rad($orig_long);
+    $i2 = deg2rad($dest_long);
     
-    $d = $self->acos(cos($l1 - $l2) - (1 - cos($i1 - $i2)) * cos($l1) * cos($l2));
+    $d = acos(cos($l1 - $l2) - (1 - cos($i1 - $i2)) * cos($l1) * cos($l2));
     # One degree of such an arc on the earth's surface is 60 international nautical miles NM
-    return $d * 60 * RtoD;
+    return rad2deg($d * 60);
 }
-#==========================================================
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #Calculates the direction from one point to another on the Earth
 # a = arccos{[sin(lat2) - cos(d + lat1 - 1.5708)]/cos(lat1)/sin(d) + 1}
 # Great Circle Bearing
@@ -107,41 +95,91 @@ sub GreatCircleDirection {
 my ($self, $orig_lat, $dest_lat, $orig_long, $dest_long, $distance) = @_;
 my ($a, $b, $d, $l1, $l2, $i1, $i2, $result, $dlong);
     
-	$l1 = $orig_lat * DtoR;
-	$l2 = $dest_lat * DtoR;
-	$d = ($distance / 60) * DtoR; # divide by 60 for nautical miles NM to degree
+	$l1 = deg2rad($orig_lat);
+	$l2 = deg2rad($dest_lat);
+	$d = deg2rad($distance / 60); # divide by 60 for nautical miles NM to degree
 
-	$i1 = $orig_long * DtoR;
-	$i2 = $dest_long * DtoR;
+	$i1 = deg2rad($orig_long);
+	$i2 = deg2rad($dest_long);
 	$dlong = $i1 - $i2;
 
-	$a = sin($l2) - cos($d + $l1 - PI / 2);
-	$b = $self->acos($a / (cos($l1) * sin($d)) + 1);
+	$a = sin($l2) - cos($d + $l1 - pi / 2);
+	$b = acos($a / (cos($l1) * sin($d)) + 1);
+	if ((abs($dlong) < pi && $dlong < 0) || (abs($dlong) > pi && $dlong > 0) ) {
+        #$b = (2 * pi) - $b;
+    }
+	$result = rad2deg($b);
 
-	#If (Abs(Dlong) < pi And Dlong < 0) Or (Abs(Dlong) > pi And Dlong > 0) Then
-	#        Result = (2 * pi) - B
-	#Else
-	#        Result = B
-	#End If
-
-	$result = $b;
-	return $result * RtoD;
+	return $result;
 }
-#==========================================================
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #The Equivalent Earth redius is 6,378.14 Kilometers.
 # Calculates the direction of the Qibla from any point on
 # the Earth From North Clocklwise
-sub QiblaDirection {
+sub QiblaDirection_ {
 my ($self, $orig_lat, $orig_long) = @_;
 my ($distance, $bearing);
     
 	# Kabah Lat=21 Deg N, Long 40 Deg E
 	$distance = $self->GreatCircleDistance($orig_lat, $self->{DestLat}, $orig_long, $self->{DestLong});
 	$bearing = $self->GreatCircleDirection($orig_lat, $self->{DestLat}, $orig_long, $self->{DestLong}, $distance);
+
+    if ($orig_lat > $self->{DestLat}) {
+        #$bearing += 180;
+    }
+
 	return $bearing;
 }
-#==========================================================
-#==========================================================
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+# Determine Qibla direction using basic spherical trigonometric formula
+# Return float Qibla Direction from the north direction in degrees
+sub QiblaDirection {
+    
+    my ($self, $orig_lat, $orig_long) = @_;
+
+    #$orig_lat = 35.3833;
+    #$orig_long = 119.0166;
+    #Saudi Arabia, Riyadh Qibla = 245,  Qibla Direction: 244.53 degees from true North,  Distance from Ka'bah in Makkah: 790.18 km
+
+    my $numerator   = sin(deg2rad($self->{DestLong} - $orig_long));
+    my $denominator = (cos(deg2rad($orig_lat)) * tan(deg2rad($self->{DestLat}))) -
+                      (sin(deg2rad($orig_lat)) * cos(deg2rad($self->{DestLong} - $orig_long)));
+
+    my $q = rad2deg(atan($numerator / $denominator));
+
+    if ($orig_lat > $self->{DestLat}) {
+        $q += 180;
+    }
+
+    return $q;
+}
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+=cuts
+public function getQibla () 
+{
+    // The geographical coordinates of the Ka'ba
+    $K_latitude  = 21.423333;
+    $K_longitude = 39.823333;
+    
+    $latitude  = $this->lat;
+    $longitude = $this->long;
+
+    $numerator   = sin(deg2rad($K_longitude - $longitude));
+    $denominator = (cos(deg2rad($latitude)) * tan(deg2rad($K_latitude))) -
+                   (sin(deg2rad($latitude)) 
+                   * cos(deg2rad($K_longitude - $longitude)));
+
+    $q = atan($numerator / $denominator);
+    $q = rad2deg($q);
+    
+    if ($this->lat > 21.423333) {
+        $q += 180;
+    }
+    
+    return $q;
+}
+    
+=cut
 
 1;
 
@@ -199,7 +237,7 @@ Website: http://www.islamware.com   http://www.mewsoft.com
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2006-2013 by Ahmed Amin Elsheshtawy support@islamware.com, support@mewsoft.com
+Copyright (C) 2006-2015 by Ahmed Amin Elsheshtawy support@islamware.com, support@mewsoft.com
 L<http://www.islamware.com>  L<http://www.mewsoft.com>
 
 This library is free software; you can redistribute it and/or modify
